/*******************************************************************************
 * Copyright Moxa Inc.
 *
 * DIO TAG Sample
 *
 * Date          Author       Comment
 * 02-21-2014    TJ Tai       Created.
 ******************************************************************************/

/*!
    \file ioPAC5542_tag_dio.c
    \copyright Moxa Inc.
    \brief <b>\a ioPAC5542 \a DIO \a TAG \a Sample  </b>
    \date 02-21-2014
    \author TJ Tai
    \version V1.0
    \image html 5542_tag_dio.jpg "DIO TAG Sample"
    \details
    <pre>
    <b>Introduction:</b>
    This DIO sample code sets the direction of DIO(0-3) to DI and DIO(4-7) to DO,
    and it reads DIO(0-3) status and output the status to DIO(4-7).
    Therefore, if you change the DIO(0) status to false, the output of DIO(4) would be false as well.
    The polling delay is 1 second.
    <b>Example:</b>
    1. Using default: ./ioPAC5542_tag_dio
    <b>Help:</b>
    </pre>
    \verbatim
    root@Moxa:/tmp#./ioPAC5542_tag_dio -h
    DIO TAG sample program.

    Usage: ./ioPAC5542_tag_dio

    \endverbatim
    \b Library: \n
    \link TagService
    TAG APIs
    \endlink
    \n\b RTUxpress \b Project \b file: \n
    <a href="./rtu/ioPAC5542_tag_dio.rtu" target="_blank"><b>ioPAC5542_tag_dio.rtu</b></a>
    \n(Please right click on the link and ‘Save Target As…’ to save RTUxpress project file and open it with RTUxpress utility)
*/

#include "libmoxa_rtu.h"

//! \include ioPAC5542_tag_dio.c

int main(int argc, char * argv[])
{
	int rc, i;
	TAG_ERR_CODE retval = 0;
	char szFileName[80] = "/etc/RTU/config.rtu";
	TAG_INFO tagInfo;
	int dioChannelAmount = 8;
	char dio_tagName[dioChannelAmount][TAG_MAX_NAME_SIZE];

	while(-1 != (rc = getopt(argc, argv, "h")))
	{
		switch(rc)
		{
			case '?':
			case 'h':
			default:
				printf("DO TAG sample program.\n\n");
				printf("Usage: ./ioPAC5542_tag_dio \n\n");
				printf("\n");
				return 0;
		}
	}


	retval = MX_RTU_Tag_Init();
	if(retval != TAG_ERR_OK)
	{
		printf("MX_RTU_Tag_Init(), return code = %d.\n", retval);
		return 0;
	}

	// Config DI TAG
	for(i = 0; i < dioChannelAmount / 2; i++)
	{
		sprintf(dio_tagName[i], "S0_DIO%d_DIValue", i);
	}

	// Config DO TAG
	for(i = dioChannelAmount / 2; i < dioChannelAmount; i++)
	{
		sprintf(dio_tagName[i], "S0_DIO%d_DOValue", i);
	}

	UINT32 bitVal[dioChannelAmount / 2];
	for(i = 0; i < dioChannelAmount / 2; i++)
	{
		bitVal[i] = 0;
	}

	while(1)
	{
		// GetDI
		for(i = 0; i < dioChannelAmount / 2 ; i++)
		{
			retval = MX_RTU_Tag_Read(dio_tagName[i], &bitVal[i], sizeof(bitVal), NULL, NULL);
			if(retval != TAG_ERR_OK)
			{
				printf("MX_RTU_Tag_Read(%s) = %d\n", dio_tagName[i], retval);
				return 0;
			}
		}

		printf("\r DI = DO ");
		fflush(0);

		// SetDO
		for(i = dioChannelAmount / 2; i < dioChannelAmount; i++)
		{
			retval = MX_RTU_Tag_Get_Info(dio_tagName[i], &tagInfo);
			if(retval != TAG_ERR_OK)
			{
				printf("MX_RTU_Tag_Get_Info(%s), return code = %d.\n", dio_tagName[i], retval);
				return 0;
			}
			retval = MX_RTU_Tag_Write(dio_tagName[i], bitVal + i - 4, tagInfo.tagSize);
			if(retval != TAG_ERR_OK)
			{
				printf("MX_RTU_Tag_Write(%s), return code = %d\r\n", dio_tagName[i], retval);
				return 0;
			}
		}
		sleep(1);
	}

	retval = MX_RTU_Tag_Uninit();
	if(retval != TAG_ERR_OK)
	{
		printf("MX_RTU_Tag_Uninit(), return code = %d\n", retval);
	}

	return 0;
}
