/*******************************************************************************
 * Copyright Moxa Inc.
 *
 * DI CGI Sample
 *
 * Date          Author            Comment
 * 11-11-2014   Wanhan Hsieh     Created.
 ******************************************************************************/

/*!
    \file di_cgi.c
    \copyright Moxa Inc.
    \brief <b>\a DI \a CGI \a Sample</b>
    \date 11-11-2014
    \author Wanhan Hsieh
    \version V1.0
    \image html di_cgi.jpg "DI CGI Sample"
    \details
    <pre>
    <b>Introduction:</b>
    This is DI CGI sample code. You can monitor DI status and control DI mode.
    <b>Example:</b>
    1. Using default: http://192.168.127.254/cgi-bin/di.cgi
    2. Setting DI slot and channel: http://192.168.127.254/cgi-bin/di.cgi?slot=2&chStart=0&chCount=16
    <b>Default:</b>
    Slot of DI module = 1
    </pre>
    \b Library: \n
    \link DI
    DI APIs
    \endlink
*/

#include <libmoxa_rtu.h>

#define CMD_SET_MODE			0

int getAuth(void)
{
 	return setuid(0);
}

void startHtml(char *title)
{
	printf("Content-type: text/html; charset=utf-8\n\n");
	printf("<html>");
	printf("<head>");
	printf("<link href=\"../iopac.css\" rel=stylesheet type=text/css>");
	printf("<title>%s</title>", title);
	printf("<script Language=\"JavaScript\">");
	printf("function refreshPage(){window.location.reload();}");
	//printf("setTimeout('refreshPage()',3000);");	// refresh page periodically
	printf("</script>");
	printf("</head>");
	printf("<body>\n");
	printf("<input type=\"button\" value=\"Refresh page\" onClick=\"refreshPage()\">");
}

void endHtml(void)
{
	printf("</body>");
	printf("</html>\n");
}

void startTable(int border, int width)
{
	printf("<table border=\"%d\" width=\"%d%%\">", border, width);
}

void endTable(void)
{
	printf("</table>");
}

void startOneRow(void)
{
	printf("<tr>");
}

void endOneRow(void)
{
	printf("</tr>");
}

void insertData(int width, char *align, char *cssClass, char *data)
{
	printf("<td width=\"%d%%\" align=\"%s\" class=\"%s\">%s</td>", width, align, cssClass, data);
}

int execCmd(UINT8 slot, UINT8 chStart, UINT8 chCount, char *cmd)
{
	int cmdIdx = -1;
	int channel = 0;
	int setValue = 0;
	int ret;

	if(cmd == NULL || strcmp(cmd, "") == 0)
	{
		//printf("Null command.\n");
		return -1;
	}
	sscanf(cmd, "cmd=%d&ch=%d&set=%d", &cmdIdx, &channel, &setValue);
	//printf("cmd=%d, ch=%d, set=%d<br>\n", cmdIdx, channel, setValue);
	switch(cmdIdx)
	{
		case CMD_SET_MODE:
		{
			int i = 0;
			UINT8 mode = (UINT8) setValue;
			ret = MX_RTU_Module_DI_Mode_Set(slot, channel, 1, &mode);
			if(ret != MODULE_RW_ERR_OK)
				printf("MX_RTU_Module_DI_Mode_Set = %d<br>\n", ret);
			if(mode == DI_MODE_COUNTER)
			{
				UINT32 counterValue = 0;
				UINT32 counterStart = (0x1 << channel);
				ret = MX_RTU_Module_DI_Counter_Value_Set(slot, channel, 1, &counterValue);
				if(ret != MODULE_RW_ERR_OK)
					printf("MX_RTU_Module_DI_Counter_Value_Set = %d<br>\n", ret);
				ret = MX_RTU_Module_DI_Counter_Start_Set(slot, counterStart);
				if(ret != MODULE_RW_ERR_OK)
					printf("MX_RTU_Module_DI_Counter_Start_Set = %d<br>\n", ret);
			}
			else if(mode == DI_MODE_FREQUENCY)
			{
				UINT32 freqValue = 0;
				UINT32 freqStart = (0x1 << channel);
				ret = MX_RTU_Module_DI_Frequency_Start_Set(slot, freqStart);
				if(ret != MODULE_RW_ERR_OK)
					printf("MX_RTU_Module_DI_Frequency_Start_Set = %d<br>\n", ret);
			}
			break;
		}
		default:
			printf("Error command.\n");
			break;
	}

	return 0;
}

void getCgiArg(UINT8 *pSlot, UINT8 *pChStart, UINT8 *pChCount, char *command)
{
	char *p;
	int slot, chStart, chCount;
	char buf[100] = {0};
	if((p = getenv("QUERY_STRING")))
	{
		if(0 == strcmp(p, ""))
		{
			//printf("Empty QUERY_STRING.\n");
			return;
		}
	}
	else
	{
		//printf("Null QUERY_STRING.\n");
		return;
	}
	sscanf(p, "slot=%d&chStart=%d&chCount=%d&%s", &slot, &chStart, &chCount, buf);
	if(buf)
		memcpy(command, buf, strlen(buf));
	*pSlot = slot;
	*pChStart = chStart;
	*pChCount = chCount;
}

//! \include di_cgi.c
int main(int argc, char* argv[], char *envp[])
{
	int i;
	int ret = 0;
	int chAmount = 16;
	UINT8 diSlot = 1, chStart = 0, chCount = 16;
	char command[100] = {0};
	UINT8 diMode[chAmount];
	UINT32 diValue = 0;
	UINT32 diCounter[chAmount];
	UINT32 diFilter[chAmount];
	UINT8 diTrigger[chAmount];
	float freqValue[chAmount];
	char *cgiName;
	cgiName = getenv("SCRIPT_NAME");

	getAuth();

	startHtml("DI CGI Sample");

	getCgiArg(&diSlot, &chStart, &chCount, command);
	if(0 == execCmd(diSlot, chStart, chCount, command))	// execute successfully, then reload page
	{
		printf("<script Language=\"JavaScript\">");
		printf("window.location.assign(\"%s?slot=%d&chStart=%d&chCount=%d\");",
				cgiName, diSlot, chStart, chCount);
		printf("</script>");
	}
	else
	{
		startTable(0, 100);
		startOneRow();
		insertData(15, "left", "block_title", "DI Channel");
		insertData(15, "center", "block_title", "Mode");
		insertData(20, "center", "block_title", "Status");
		insertData(20, "center", "block_title", "Filter");
		insertData(20, "center", "block_title", "Counter Trigger");
		endOneRow();

		ret = MX_RTU_Module_DI_Mode_Get(diSlot, chStart, chCount, diMode);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_DI_Mode_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_DI_Value_Get(diSlot, &diValue, NULL);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_DI_Value_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_DI_Counter_Value_Get(diSlot, chStart, chCount, diCounter, NULL);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_DI_Counter_Value_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_DI_Filter_Get(diSlot, chStart, chCount, diFilter);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_DI_Filter_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_DI_Counter_Trigger_Get(diSlot, chStart, chCount, diTrigger);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_DI_Counter_Trigger_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_DI_Frequency_Get(diSlot, chStart, chCount, freqValue);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_DI_Frequency_Get = %d<br>\n", ret);
		for(i = 0; i < chCount; i++)
		{
			char chMsg[20], modeMsg[100], statusMsg[20], filterMsg[20], triggerMsg[20];
			char cssClass[20];
			if(i % 2)
				sprintf(cssClass, "column_title_bg");
			else
				sprintf(cssClass, "column_title");
			sprintf(chMsg, "DI-%02d", chStart+i);
			sprintf(filterMsg, "%d", diFilter[i]);
			if(diMode[i] == DI_MODE_DI)
			{
				sprintf(modeMsg, "<a href=\"%s?slot=%d&chStart=%d&chCount=%d&cmd=%d&ch=%d&set=%d\">%s</a>",
					cgiName, diSlot, chStart, chCount, CMD_SET_MODE, chStart+i, DI_MODE_COUNTER, "DI");
				sprintf(statusMsg, "%s", ((diValue >> (chStart+i)) & 0x1) ? "ON" : "OFF");
				sprintf(triggerMsg, "--");
			}
			else if(diMode[i] == DI_MODE_COUNTER)
			{
				sprintf(modeMsg, "<a href=\"%s?slot=%d&chStart=%d&chCount=%d&cmd=%d&ch=%d&set=%d\">%s</a>",
					cgiName, diSlot, chStart, chCount, CMD_SET_MODE, chStart+i, DI_MODE_FREQUENCY, "Counter");
				sprintf(statusMsg, "%d", diCounter[i]);
				sprintf(triggerMsg, "%s", (diTrigger[i] == DI_TOGGLE_L2H) ? "Rising" :
									(diTrigger[i] == DI_TOGGLE_H2L) ? "Falling" : "Both");
			}
			else if(diMode[i] == DI_MODE_FREQUENCY)
			{
				sprintf(modeMsg, "<a href=\"%s?slot=%d&chStart=%d&chCount=%d&cmd=%d&ch=%d&set=%d\">%s</a>",
					cgiName, diSlot, chStart, chCount, CMD_SET_MODE, chStart+i, DI_MODE_DI, "Frequency");
				sprintf(statusMsg, "%.1f", freqValue[i]);
				sprintf(triggerMsg, "--");
			}
			startOneRow();
			insertData(15, "left", cssClass, chMsg);	// channel
			insertData(15, "center", cssClass, modeMsg);	// mode
			insertData(20, "center", cssClass, statusMsg);	// status
			insertData(20, "center", cssClass, filterMsg);	// filter
			insertData(20, "center", cssClass, triggerMsg);	// trigger
			endOneRow();
		}
		endTable();
		endHtml();
	}
	return ret;
}

