/*******************************************************************************
 * Copyright Moxa Inc.
 *
 * RTD CGI Sample
 *
 * Date          Author            Comment
 * 11-11-2014   Wanhan Hsieh     Created.
 ******************************************************************************/

/*!
    \file rtd_cgi.c
    \copyright Moxa Inc.
    \brief <b>\a RTD \a CGI \a Sample</b>
    \date 11-11-2014
    \author Wanhan Hsieh
    \version V1.0
    \image html rtd_cgi.jpg "RTD CGI Sample"
    \details
    <pre>
    <b>Introduction:</b>
    This is RTD CGI sample code. You can monitor RTD status and control RTD mode.
    <b>Example:</b>
    1. Using default: http://192.168.127.254/cgi-bin/rtd.cgi
    2. Setting RTD slot and channel: http://192.168.127.254/cgi-bin/rtd.cgi?slot=2&chStart=0&chCount=16
    <b>Default:</b>
    Slot of RTD module = 1
    </pre>
    \b Library: \n
    \link RTD
    RTD APIs
    \endlink
*/

#include <libmoxa_rtu.h>

#define CMD_SET_TYPE			0

char *rtdTypeList[] =
{
	"2200ohm",
	"1250ohm",
	"620ohm",
	"310ohm",
	"PT50",
	"PT100",
	"PT200",
	"PT500",
	"PT1000",
	"JPT100",
	"JPT200",
	"JPT500",
	"JPT1000",
	"NI100",
	"NI200",
	"NI500",
	"NI1000",
	"NI120"
};

int getAuth(void)
{
 	return setuid(0);
}

void startHtml(char *title)
{
	printf("Content-type: text/html; charset=utf-8\n\n");
	printf("<html>");
	printf("<head>");
	printf("<link href=\"../iopac.css\" rel=stylesheet type=text/css>");
	printf("<title>%s</title>", title);
	printf("<script Language=\"JavaScript\">");
	printf("function refreshPage(){window.location.reload();}");
	//printf("setTimeout('refreshPage()',3000);");	// refresh page periodically
	printf("</script>");
	printf("</head>");
	printf("<body>\n");
	printf("<input type=\"button\" value=\"Refresh page\" onClick=\"refreshPage()\">");
}

void endHtml(void)
{
	printf("</body>");
	printf("</html>\n");
}

void startTable(int border, int width)
{
	printf("<table border=\"%d\" width=\"%d%%\">", border, width);
}

void endTable(void)
{
	printf("</table>");
}

void startOneRow(void)
{
	printf("<tr>");
}

void endOneRow(void)
{
	printf("</tr>");
}

void insertData(int width, char *align, char *cssClass, char *data)
{
	printf("<td width=\"%d%%\" align=\"%s\" class=\"%s\">%s</td>", width, align, cssClass, data);
}

int execCmd(UINT8 slot, UINT8 chStart, UINT8 chCount, char *cmd)
{
	int cmdIdx = -1;
	int channel = 0;
	int setValue = 0;
	int ret;

	if(cmd == NULL || strcmp(cmd, "") == 0)
	{
		//printf("Null command.\n");
		return -1;
	}
	sscanf(cmd, "cmd=%d&ch=%d&set=%d", &cmdIdx, &channel, &setValue);
	//printf("cmd=%d, ch=%d, set=%d<br>\n", cmdIdx, channel, setValue);
	switch(cmdIdx)
	{
		case CMD_SET_TYPE:
		{
			int i = 0;
			UINT8 type = (UINT8) setValue;
			ret = MX_RTU_Module_RTD_Type_Set(slot, channel, 1, &type);
			if(ret != MODULE_RW_ERR_OK)
				printf("MX_RTU_Module_RTD_Type_Set = %d<br>\n", ret);
			break;
		}
		default:
			printf("Error command.\n");
			break;
	}
	
	return 0;
}

void getCgiArg(UINT8 *pSlot, UINT8 *pChStart, UINT8 *pChCount, char *command)
{
	char *p;
	int slot, chStart, chCount;
	char buf[100] = {0};
	if((p = getenv("QUERY_STRING")))
	{
		if(0 == strcmp(p, ""))
		{
			//printf("Empty QUERY_STRING.\n");
			return;
		}
	}
	else
	{
		//printf("Null QUERY_STRING.\n");
		return;
	}
	sscanf(p, "slot=%d&chStart=%d&chCount=%d&%s", &slot, &chStart, &chCount, buf);
	if(buf)
		memcpy(command, buf, strlen(buf));
	*pSlot = slot;
	*pChStart = chStart;
	*pChCount = chCount;
}

//! \include rtd_cgi.c
int main(int argc, char* argv[], char *envp[])
{
	int i, j;
	int ret = 0;
	int chAmount = 6;
	UINT8 rtdSlot = 1, chStart = 0, chCount = 6;
	char command[100] = {0};
	UINT8 rtdType[chAmount];
	UINT8 rtdStatus[chAmount];
	float rtdValue[chAmount];
	float rtdMin[chAmount];
	float rtdMax[chAmount];
	char *cgiName;
	cgiName = getenv("SCRIPT_NAME");

	getAuth();
		
	startHtml("RTD CGI Sample");

	getCgiArg(&rtdSlot, &chStart, &chCount, command);
	if(0 == execCmd(rtdSlot, chStart, chCount, command))	// execute successfully, then reload page
	{
		printf("<script Language=\"JavaScript\">");
		printf("window.location.assign(\"%s?slot=%d&chStart=%d&chCount=%d\");",
				cgiName, rtdSlot, chStart, chCount);
		printf("</script>");
	}
	else
	{
		startTable(0, 100);
		startOneRow();
		insertData(15, "left", "block_title", "RTD Channel");
		insertData(15, "center", "block_title", "Sensor Type");
		insertData(20, "center", "block_title", "Value");
		insertData(20, "center", "block_title", "Min");
		insertData(20, "center", "block_title", "Max");
		endOneRow();

		ret = MX_RTU_Module_RTD_Type_Get(rtdSlot, chStart, chCount, rtdType);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_RTD_Type_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_RTD_Eng_Value_Get(rtdSlot, chStart, chCount, rtdValue, NULL);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_RTD_Eng_Value_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_RTD_Burnout_Status_Get(rtdSlot, chStart, chCount, rtdStatus);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_RTD_Burnout_Status_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_RTD_Min_Eng_Get(rtdSlot, chStart, chCount, rtdMin);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_RTD_Min_Eng_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_RTD_Max_Eng_Get(rtdSlot, chStart, chCount, rtdMax);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_RTD_Max_Eng_Get = %d<br>\n", ret);
		for(i = 0; i < chCount; i++)
		{
			char chMsg[20], typeMsg[100], valueMsg[20], minMsg[20], maxMsg[20];
			char cssClass[20];
			if(i % 2)
				sprintf(cssClass, "column_title_bg");
			else
				sprintf(cssClass, "column_title");
			sprintf(chMsg, "RTD-%02d", chStart+i);
			sprintf(minMsg, "%.3f", rtdMin[i]);
			sprintf(maxMsg, "%.3f", rtdMax[i]);
			if(rtdStatus[i] == BURNOUT_STATUS_BURNOUT)
				sprintf(valueMsg, "Burn Out");
			else
				sprintf(valueMsg, "%.1f", rtdValue[i]);
			startOneRow();
			insertData(15, "left", cssClass, chMsg);	// channel
			// type 
			printf("<td width=\"%d%%\" align=\"%s\" class=\"%s\">", 15, "center", cssClass);		
			printf("<select onChange=\"location = this.options[this.selectedIndex].value;\">");
			printf("<option value=\"#\">%s</option>", rtdTypeList[rtdType[i]]);
			for(j = 0; j < sizeof(rtdTypeList)/sizeof(rtdTypeList[0]); j++)
			{
				printf("<option value=\"%s?slot=%d&chStart=%d&chCount=%d&cmd=%d&ch=%d&set=%d\">%s</option>",
					cgiName, rtdSlot, chStart, chCount, CMD_SET_TYPE, chStart+i, j, rtdTypeList[j]);
			}
			printf("</select></td>");
			insertData(20, "center", cssClass, valueMsg);	// value
			insertData(20, "center", cssClass, minMsg);	// min
			insertData(20, "center", cssClass, maxMsg);	// max
			endOneRow();
		}
		endTable();
		endHtml();
	}
	return ret;
}

