/*******************************************************************************
 * Copyright Moxa Inc.
 *
 * TC CGI Sample
 *
 * Date          Author            Comment
 * 11-11-2014   Wanhan Hsieh     Created.
 ******************************************************************************/

/*!
    \file tc_cgi.c
    \copyright Moxa Inc.
    \brief <b>\a TC \a CGI \a Sample</b>
    \date 11-11-2014
    \author Wanhan Hsieh
    \version V1.0
    \image html tc_cgi.jpg "TC CGI Sample"
    \details
    <pre>
    <b>Introduction:</b>
    This is TC CGI sample code. You can monitor TC status and control TC mode.
    <b>Example:</b>
    1. Using default: http://192.168.127.254/cgi-bin/tc.cgi
    2. Setting TC slot and channel: http://192.168.127.254/cgi-bin/tc.cgi?slot=2&chStart=0&chCount=16
    <b>Default:</b>
    Slot of TC module = 1
    </pre>
    \b Library: \n
    \link TC
    TC APIs
    \endlink
*/

#include <libmoxa_rtu.h>

#define CMD_SET_TYPE			0

char *tcTypeList[] =
{
	"78.126mV",
	"39.062mV",
	"19.532mV",
	"J Tpye",
	"K Tpye",
	"T Tpye",
	"E Tpye",
	"R Tpye",
	"S Tpye",
	"B Tpye",
	"N Tpye"
};

int getAuth(void)
{
 	return setuid(0);
}

void startHtml(char *title)
{
	printf("Content-type: text/html; charset=utf-8\n\n");
	printf("<html>");
	printf("<head>");
	printf("<link href=\"../iopac.css\" rel=stylesheet type=text/css>");
	printf("<title>%s</title>", title);
	printf("<script Language=\"JavaScript\">");
	printf("function refreshPage(){window.location.reload();}");
	//printf("setTimeout('refreshPage()',3000);");	// refresh page periodically
	printf("</script>");
	printf("</head>");
	printf("<body>\n");
	printf("<input type=\"button\" value=\"Refresh page\" onClick=\"refreshPage()\">");
}

void endHtml(void)
{
	printf("</body>");
	printf("</html>\n");
}

void startTable(int border, int width)
{
	printf("<table border=\"%d\" width=\"%d%%\">", border, width);
}

void endTable(void)
{
	printf("</table>");
}

void startOneRow(void)
{
	printf("<tr>");
}

void endOneRow(void)
{
	printf("</tr>");
}

void insertData(int width, char *align, char *cssClass, char *data)
{
	printf("<td width=\"%d%%\" align=\"%s\" class=\"%s\">%s</td>", width, align, cssClass, data);
}

int execCmd(UINT8 slot, UINT8 chStart, UINT8 chCount, char *cmd)
{
	int cmdIdx = -1;
	int channel = 0;
	int setValue = 0;
	int ret;
	
	if(cmd == NULL || strcmp(cmd, "") == 0)
	{
		//printf("Null command.\n");
		return -1;
	}
	sscanf(cmd, "cmd=%d&ch=%d&set=%d", &cmdIdx, &channel, &setValue);
	//printf("cmd=%d, ch=%d, set=%d<br>\n", cmdIdx, channel, setValue);
	switch(cmdIdx)
	{
		case CMD_SET_TYPE:
		{
			int i = 0;
			UINT8 type = (UINT8) setValue;
			ret = MX_RTU_Module_TC_Type_Set(slot, channel, 1, &type);
			if(ret != MODULE_RW_ERR_OK)
				printf("MX_RTU_Module_TC_Type_Set = %d<br>\n", ret);
			break;
		}
		default:
			printf("Error command.\n");
			break;
	}
	
	return 0;
}

void getCgiArg(UINT8 *pSlot, UINT8 *pChStart, UINT8 *pChCount, char *command)
{
	char *p;
	int slot, chStart, chCount;
	char buf[100] = {0};
	if((p = getenv("QUERY_STRING")))
	{
		if(0 == strcmp(p, ""))
		{
			//printf("Empty QUERY_STRING.\n");
			return;
		}
	}
	else
	{
		//printf("Null QUERY_STRING.\n");
		return;
	}
	sscanf(p, "slot=%d&chStart=%d&chCount=%d&%s", &slot, &chStart, &chCount, buf);
	if(buf)
		memcpy(command, buf, strlen(buf));
	*pSlot = slot;
	*pChStart = chStart;
	*pChCount = chCount;
}

//! \include tc_cgi.c
int main(int argc, char* argv[], char *envp[])
{
	int i, j;
	int ret = 0;
	int chAmount = 6;
	UINT8 tcSlot = 1, chStart = 0, chCount = 8;
	char command[100] = {0};
	UINT8 tcType[chAmount];
	UINT8 tcStatus[chAmount];
	float tcValue[chAmount];
	float tcMin[chAmount];
	float tcMax[chAmount];
	char *cgiName;
	cgiName = getenv("SCRIPT_NAME");

	getAuth();
		
	startHtml("TC CGI Sample");

	getCgiArg(&tcSlot, &chStart, &chCount, command);
	if(0 == execCmd(tcSlot, chStart, chCount, command))	// execute successfully, then reload page
	{
		printf("<script Language=\"JavaScript\">");
		printf("window.location.assign(\"%s?slot=%d&chStart=%d&chCount=%d\");",
				cgiName, tcSlot, chStart, chCount);
		printf("</script>");
	}
	else
	{
		startTable(0, 100);
		startOneRow();
		insertData(15, "left", "block_title", "TC Channel");
		insertData(15, "center", "block_title", "Sensor Type");
		insertData(20, "center", "block_title", "Value");
		insertData(20, "center", "block_title", "Min");
		insertData(20, "center", "block_title", "Max");
		endOneRow();

		ret = MX_RTU_Module_TC_Type_Get(tcSlot, chStart, chCount, tcType);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_TC_Type_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_TC_Eng_Value_Get(tcSlot, chStart, chCount, tcValue, NULL);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_TC_Eng_Value_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_TC_Burnout_Status_Get(tcSlot, chStart, chCount, tcStatus);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_TC_Burnout_Status_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_TC_Min_Eng_Get(tcSlot, chStart, chCount, tcMin);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_TC_Min_Eng_Get = %d<br>\n", ret);
		ret = MX_RTU_Module_TC_Max_Eng_Get(tcSlot, chStart, chCount, tcMax);
		if(ret != MODULE_RW_ERR_OK)
			printf("MX_RTU_Module_TC_Max_Eng_Get = %d<br>\n", ret);
		for(i = 0; i < chCount; i++)
		{
			char chMsg[20], typeMsg[100], valueMsg[20], minMsg[20], maxMsg[20];
			char cssClass[20];
			if(i % 2)
				sprintf(cssClass, "column_title_bg");
			else
				sprintf(cssClass, "column_title");
			sprintf(chMsg, "TC-%02d", chStart+i);
			sprintf(minMsg, "%.3f", tcMin[i]);
			sprintf(maxMsg, "%.3f", tcMax[i]);
			if(tcStatus[i] == BURNOUT_STATUS_BURNOUT)
				sprintf(valueMsg, "Burn Out");
			else
				sprintf(valueMsg, "%.1f", tcValue[i]);
			startOneRow();
			insertData(15, "left", cssClass, chMsg);	// channel
			// type 
			printf("<td width=\"%d%%\" align=\"%s\" class=\"%s\">", 15, "center", cssClass);		
			printf("<select onChange=\"location = this.options[this.selectedIndex].value;\">");
			printf("<option value=\"#\">%s</option>", tcTypeList[tcType[i]]);
			for(j = 0; j < sizeof(tcTypeList)/sizeof(tcTypeList[0]); j++)
			{
				printf("<option value=\"%s?slot=%d&chStart=%d&chCount=%d&cmd=%d&ch=%d&set=%d\">%s</option>",
					cgiName, tcSlot, chStart, chCount, CMD_SET_TYPE, chStart+i, j, tcTypeList[j]);
			}
			printf("</select></td>");
			insertData(20, "center", cssClass, valueMsg);	// value
			insertData(20, "center", cssClass, minMsg);	// min
			insertData(20, "center", cssClass, maxMsg);	// max
			endOneRow();
		}
		endTable();
		endHtml();
	}
	return ret;
}

